@@ -32,11 +32,9 @@ explicit tasks.
 """
 
 import abc
-import six
 
 
-@six.add_metaclass(abc.ABCMeta)
-class JobProvider(object):
+class JobProvider(object, metaclass=abc.ABCMeta):
   """Interface all job providers should inherit from."""
 
   # A member field that a provider can use to expose a status message
@@ -147,8 +145,7 @@ class JobProvider(object):
                        create_time_min=None,
                        create_time_max=None,
                        max_tasks=0,
-                       page_size=0,
-                       verbose=True):
+                       page_size=0):
     """Return a list of tasks based on the search criteria.
 
     If any of the filters are empty or {'*'}, then no filtering is performed on
@@ -172,7 +169,6 @@ class JobProvider(object):
       max_tasks: the maximum number of job tasks to return or 0 for no limit.
       page_size: the page size to use for each query to the backend. May be
                  ignored by provider implementations.
-      verbose: if set to true, will output retrying error messages
 
     Returns:
       A list of Task objects.
