@@ -52,8 +52,6 @@ Thus using the local runner requires:
 Note that the local runner supports the `--tasks` parameter. All tasks
 submitted will run concurrently.
 """
-from __future__ import print_function
-
 import collections
 import datetime
 import os
@@ -472,9 +470,8 @@ class LocalJobProvider(base.JobProvider):
       create_time_min=None,
       create_time_max=None,
       max_tasks=0,
-      # page_size and verbose are ignored for the LocalJobProvider
-      page_size=0,
-      verbose=True):
+      # page_size is ignored for the LocalJobProvider
+      page_size=0):
 
     # 'OR' filtering arguments.
     statuses = None if statuses == {'*'} else statuses
